package com.simuwang.manage.service.impl;

import cn.hutool.core.date.DateUtil;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.DistributionMapper;
import com.simuwang.base.mapper.NavMapper;
import com.simuwang.base.pojo.dos.DistributionDO;
import com.simuwang.base.pojo.dos.DistributionTablePageDO;
import com.simuwang.base.pojo.dos.NavDO;
import com.simuwang.base.pojo.dto.query.DistributionPageQuery;
import com.simuwang.base.pojo.vo.DistributionTablePageVO;
import com.simuwang.base.pojo.vo.DistributionVO;
import com.simuwang.base.pojo.vo.IdListVO;
import com.simuwang.manage.service.DistributionService;
import com.smppw.common.pojo.ResultVo;
import org.apache.poi.ss.formula.functions.Na;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * FileName: DistributionServiceImpl
 * Author:   chenjianhua
 * Date:     2024/9/15 14:07
 * Description: ${DESCRIPTION}
 */
@Service
public class DistributionServiceImpl implements DistributionService {

    @Autowired
    private DistributionMapper distributionMapper;

    @Autowired
    private NavMapper navMapper;
    @Override
    public MybatisPage<DistributionTablePageVO> searchDistributionList(DistributionPageQuery distributionPageQuery) {
        List<DistributionTablePageDO> distributionTablePageDOList = distributionMapper.searchDistributionList(distributionPageQuery);
        List<DistributionTablePageVO> distributionTablePageVOList = distributionTablePageDOList.stream().map(DistributionTablePageDO::toVo).collect(Collectors.toList());
        long total = distributionMapper.countDistributionList(distributionPageQuery);
        return MybatisPage.of(total,distributionTablePageVOList);
    }

    @Override
    public ResultVo saveDistribution(DistributionVO distributionVO) {
        ResultVo vo = new ResultVo(true);
        DistributionDO distributionDO = new DistributionDO();
        distributionDO.setFundId(distributionVO.getFundId());
        distributionDO.setDistribution(distributionVO.getDistribution());
        distributionDO.setDistributeDate(DateUtils.parse(distributionVO.getDistributeDate(),DateUtils.YYYY_MM_DD));
        distributionDO.setDistributeType(distributionVO.getDistributeType());
        distributionDO.setIsvalid(1);
        distributionDO.setUpdateTime(DateUtils.getNowDate());
        DistributionDO oldDistributionDO = distributionMapper.selectDistributionByDate(distributionVO.getFundId(),DateUtils.parse(distributionVO.getDistributeDate(),DateUtils.YYYY_MM_DD));
        if(!StringUtil.isNull(oldDistributionDO)){
            if(!oldDistributionDO.getId().equals(distributionVO.getId())){
                vo.setData(false);
                vo.setMsg("同一基金在同一天不能已存在分红数据");
                return vo;
            }
            distributionDO.setId(oldDistributionDO.getId());
            distributionMapper.updateDistributionById(distributionDO);
        }else{
            distributionDO.setCreateTime(DateUtils.getNowDate());
            distributionMapper.saveDistribution(distributionDO);
        }
        //保存净值
        NavDO navDO = new NavDO();
        navDO.setFundId(distributionVO.getFundId());
        navDO.setNav(distributionVO.getNav());
        navDO.setCumulativeNavWithdrawal(distributionVO.getCumulativeNavWithdrawal());
        navDO.setPriceDate(DateUtils.parse(distributionVO.getDistributeDate(),DateUtils.YYYY_MM_DD));
        NavDO oldNav = navMapper.queryFundNav(navDO);
        if(StringUtil.isNull(oldNav)){
            navDO.setUpdateTime(DateUtils.getNowDate());
            navDO.setCreateTime(DateUtils.getNowDate());
            navDO.setIsvalid(1);
            navMapper.saveNav(navDO);
        }else{
            navDO.setId(oldNav.getId());
            navDO.setUpdateTime(DateUtils.getNowDate());
            navMapper.updateNav(navDO);
        }
        return vo;
    }

    @Override
    public void deleteDistribution(IdListVO idListVO) {
        distributionMapper.deleteDistribution(idListVO.getIdList());
    }
}
